import pytest
from pydantic import BaseModel, ValidationError

from cuenca_validations.types import RelatedTransaction


class Model(BaseModel):
    related_transaction_uri: RelatedTransaction


def test_related_transaction():
    transaction_uri = '/deposits/SPXXX'
    model = Model(related_transaction_uri=transaction_uri)
    assert model.related_transaction_uri == transaction_uri
    assert model.related_transaction_uri.get_model() == 'Deposit'
    assert model.related_transaction_uri.id == 'SPXXX'


def test_related_transaction_with_underscore():
    transaction_uri = '/bill_payments/STXXX'
    model = Model(related_transaction_uri=transaction_uri)
    assert model.related_transaction_uri == transaction_uri
    assert model.related_transaction_uri.get_model() == 'BillPayment'
    assert model.related_transaction_uri.id == 'STXXX'


def test_invalid_uri_related_transaction():
    transaction_uri = '/depositsXXXXX'
    with pytest.raises(ValidationError) as exc_info:
        Model(related_transaction_uri=transaction_uri)
    assert exc_info.value.errors()[0] == dict(
        loc=('related_transaction_uri',),
        type='value_error',
        msg='invalid uri format',
    )



from typing import Optional, Union

from clabe import Clabe
from pydantic import BaseModel, Extra, Field, StrictStr, root_validator

from ..types.enums import CardStatus
from ..typing import DictStrAny
from .card import PaymentCardNumber, StrictPaymentCardNumber
from .general import StrictPositiveInt


class BaseRequest(BaseModel):
    class Config:
        extra = Extra.forbid

    def dict(self, *args, **kwargs) -> DictStrAny:
        kwargs.setdefault('exclude_none', True)
        kwargs.setdefault('exclude_unset', True)
        return super().dict(*args, **kwargs)


class TransferRequest(BaseRequest):
    recipient_name: StrictStr
    account_number: Union[Clabe, PaymentCardNumber]
    amount: StrictPositiveInt  # in centavos
    descriptor: StrictStr  # how it'll appear for the recipient
    idempotency_key: str  # must be unique for each transfer


class StrictTransferRequest(TransferRequest):
    account_number: Union[Clabe, StrictPaymentCardNumber]


class CardUpdateRequest(BaseRequest):
    user_id: Optional[str]
    ledger_account_id: Optional[str]
    status: Optional[CardStatus]


class CardRequest(BaseRequest):
    user_id: str
    ledger_account_id: str


class ApiKeyUpdateRequest(BaseRequest):
    user_id: Optional[str]
    metadata: Optional[DictStrAny]


class UserCredentialUpdateRequest(BaseModel):
    is_active: Optional[bool]
    password: Optional[str] = Field(
        None, max_length=6, min_length=6, regex=r'\d{6}'
    )

    class Config:
        extra = Extra.forbid

    @root_validator(pre=True)
    def check_one_property_at_a_time(cls, values: DictStrAny) -> DictStrAny:
        not_none_count = sum(1 for val in values.values() if val)
        if not_none_count > 1:
            raise ValueError('Only one property can be updated at a time')
        return values


class UserCredentialRequest(BaseRequest):
    password: str = Field(..., max_length=6, min_length=6, regex=r'\d{6}')



import re
from typing import Optional, Tuple

mapper = dict(
    bill_payments='BillPayment',
    card_transactions='CardTransaction',
    commissions='Commission',
    deposits='Deposit',
    transfers='Transfer',
    whatsapp_transfers='WhatsappTransfer',
)


class RelatedTransaction(str):
    uri: str
    resource: Optional[str]
    id: Optional[str]

    def __init__(cls, uri: str):
        cls.uri = uri
        cls.resource, cls.id = cls._get_resource_data(uri)

    @classmethod
    def __get_validators__(cls):
        yield cls
        yield cls.validate

    @classmethod
    def validate(cls, tr: 'RelatedTransaction') -> 'RelatedTransaction':
        if not tr.resource:
            raise ValueError('invalid uri format')
        return tr

    @staticmethod
    def _get_resource_data(uri: str) -> Tuple[Optional[str], Optional[str]]:
        match = re.search(r'/([a-z_]+)/([\w]+)', uri)
        if not match:
            return (None, None)
        return (match.group(1), match.group(2))

    def get_model(cls):
        return mapper[cls.resource] if cls.resource in mapper else None
